#! /usr/bin/env python
#
# Author: Jean-Marc Legrand
# Date:   Tue Mar 23 2010
#
# Copyright (C) 2010 Paraxip Technologies
# All Rights Reserved.
#
# This code is Paraxip Technologies Confidential Property.
# Use of and Access to this code is covered by a previously executed
# Non-disclosure Agreement between Paraxip Technologies and the Recipient.
# This code is being supplied for evaluation purposes only and is not to be
# used for any other purpose.  
#

import sys
import shlex
import os
import tempfile
import datetime
import time

from textwrap import TextWrapper
from optparse import OptionParser, OptParseError, OptionError, OptionConflictError, OptionValueError, BadOptionError

# Force declaration of class attributes in __init__ method
__pychecker__ = "initattr"

# Init logging
from paraxip.infra.PyLoggingFunctions import *
from paraxip.infra.Assert import *
fileScopeLogger = Logger("netborder.GwCallDataViewer")

# Import GW call data
from paraxip.gateway.GWCallData import *

# GWCallDataViewer CLI...
import cmd
class GWCallDataViewer(cmd.Cmd):
    """GW Call Data viewer."""
    
    prompt = 'gw-call-data-viewer: '
    
    def __init__(self, in_callDataRegistry,
                 completekey='tab', stdin=None, stdout=None, noprompt=False):
        cmd.Cmd.__init__(self,completekey, stdin, stdout)
        self.m_callDataRegistry = in_callDataRegistry        
        if noprompt is True:
              self.__class__.prompt = ""

    def do_quit(self, line):
        """Quit the application."""
        return True

    def do_q(self, line):
        """Quit the application."""
        return True

    def do_EOF(self, line):
        """Quit the application."""
        return True

    def parseTime( self, in_strTime ):
        
        # Expected time string: 2005-09-01 12:30:09
        timeFormat = "%Y-%m-%d %H:%M:%S"        
        return datetime.datetime. \
               fromtimestamp(time.mktime(time.strptime(in_strTime,
                                                       timeFormat)))     

    def basic_optparser(self, in_usage ):
        parser = OptionParser(usage=in_usage, add_help_option=False)
        parser.add_option( "-e", "--end-time", type="string",
                           dest="endTime",
                           help= "List all calls that end not later than endTime. Time in format: %Y-%m-%d %H:%M:%S" )
        parser.add_option( "-s", "--start-time", type="string",
                           dest="startTime",
                           help= "List all calls that started not sooner startTime.  Time in format: %Y-%m-%d %H:%M:%S" )
        parser.add_option( "-n", "--max-num-calls", type="int",
                           dest="maxNumCalls",
                           help= "Maximum number of calls to list.  This will ist the most recent calls" )
        parser.add_option( "-d", "--direction", type="choice",
                           choices=("ALL", "IN", "OUT" ),
                           dest="direction", default="ALL",
                           help= "List all calls mathing the specified directions: IN=PSTN IN, OUT=PSTN OUT, ALL=PSTN IN/OUT." )
        return parser

    def parseCmd( self, line, parser ):
        try:
            (options, positional) = self.optparser_ls().parse_args(shlex.split( line ))
            if options.startTime is not None:
                options.startTime = self.parseTime( options.startTime )
                
            if options.endTime is not None:                
                options.endTime = self.parseTime( options.endTime )

            return (options, positional)
                            
        except BadOptionError, e:
            raise ValueError( "bad option: %s" % e )
            
        except OptionError, e:
            raise ValueError( "option error: %s" % e )

        except OptionConflictError, e:
            raise ValueError( "option conflict: %s" % e )

        except OptionValueError, e:
            raise ValueError( "option value error: %s" % e )

        except OptParseError, e:
            raise ValueError( "option parse error: %s" % e )

        except ValueError, e:
            raise ValueError( "invalid date time value: %s" % e )
        
        except SystemExit, e:
            raise ValueError( "invalid option" )

    def getCallsWithDirectionFromQuery(self, in_query, in_strDirection ):
        if in_strDirection == "ALL":
            return in_query

        if in_strDirection == "IN":
            return self.m_callDataRegistry.getPstnInCallsFromQuery( in_query )

        if in_strDirection == "OUT":
            return self.m_callDataRegistry.getPstnOutCallsFromQuery( in_query )


    # lsall command methods
    def optparser_ls(self):
        usage = "usage: lsall [options]\n" \
                "List all calls according to the specified options\n" \
                "\n" \
                "Examples:\n" \
                "1) List all calls:\n" \
                "     ls\n\n" \
                "2) List all PSTN IN calls:\n" \
                "     ls -d IN\n\n" \
                "3) List all PSTN OUT calls:\n" \
                "     ls -d OUT\n\n" \
                "4) List all calls started not sooner than \"2010-01-01 00:00:00\" and ended not later than \"2010-01-02 00:00:00\":\n" \
                "     ls -s \"2010-01-01 00:00:00\" -e \"2010-01-02 00:00:00\"\n\n"
        
        return self.basic_optparser(usage)

    def help_ls(self):
        self.optparser_ls().print_help()
        self.stdout.write( "\n" )
        return False    

    def do_ls(self, line):
        try:
            (options, positional) = self.parseCmd( line,
                                                   self.optparser_ls() )
            
            session = self.m_callDataRegistry.getNewDBSession()
            allCallsQuery = self.m_callDataRegistry.getAllCallsQuery(session)
            callsWithDirectionQuery = self. \
                                      getCallsWithDirectionFromQuery(allCallsQuery,
                                                                     options.direction)
            callsInTimeRangeQuery = self.m_callDataRegistry. \
                                    getCallsInTimeRangeFromQuery( callsWithDirectionQuery,
                                                                  options.startTime,
                                                                  options.endTime)
            if options.maxNumCalls is None:
                printCallDataList( callsInTimeRangeQuery.all(), self.stdout )
            else:
                options.maxNumCalls = -1 * options.maxNumCalls
                printCallDataList( callsInTimeRangeQuery. \
                                   order_by(CallData.m_startTimestamp)[options.maxNumCalls:],
                                   self.stdout )
            
        except ValueError, e:
            log_error( fileScopeLogger,
                       "failed to parse command options \"%s\"",
                       line )
            
        return False

    # lsactive command methods
    def optparser_lsactive(self):
        usage = "usage: lsactive [options]\n" + \
                "\n".join( textWrapper.wrap( "List active calls according to the specified options. A call data entry is marked as inactive when all references to this call data are released.  For instance, a SIP call will keep a reference on the call data for 180 second after the call is DISCONNECTED.  This explains why the lsactive command shows DISCONNECTED calls that are still active." ) ) + \
                "\n\n" \
                "Examples:\n" \
                "1) List active calls:\n" \
                "     lsactive\n\n" \
                "2) List active PSTN IN calls:\n" \
                "     lsactive -d IN\n\n" \
                "3) List all PSTN OUT calls:\n" \
                "     lsactive -d OUT\n\n" \
                "4) List active calls started not sooner than \"2010-01-01 00:00:00\" and ended not later than \"2010-01-02 00:00:00\":\n" \
                "     lsactive -s \"2010-01-01 00:00:00\" -e \"2010-01-02 00:00:00\"\n\n"
        return self.basic_optparser(usage)
    
    def help_lsactive(self):
        self.optparser_lsactive().print_help()
        self.stdout.write( "\n" )
        return False    

    def do_lsactive(self, line):
        try:
            (options, positional) = self.parseCmd( line,
                                                   self.optparser_lsactive() )
            
            session = self.m_callDataRegistry.getNewDBSession()
            allCallsQuery = self.m_callDataRegistry.getActiveCallsQuery(session)
            callsWithDirectionQuery = self. \
                                      getCallsWithDirectionFromQuery(allCallsQuery,
                                                                     options.direction)
            callsInTimeRangeQuery = self.m_callDataRegistry. \
                                    getCallsInTimeRangeFromQuery( callsWithDirectionQuery,
                                                                  options.startTime,
                                                                  options.endTime)
            if options.maxNumCalls is None:
                printCallDataList( callsInTimeRangeQuery.all(), self.stdout )
            else:
                options.maxNumCalls = -1 * options.maxNumCalls
                printCallDataList( callsInTimeRangeQuery. \
                                   order_by(CallData.m_startTimestamp)[options.maxNumCalls:],
                                   self.stdout )
            
        except ValueError, e:
            log_error( fileScopeLogger,
                       "failed to parse command options \"%s\"",
                       line )
            
        return False

    # lsblocked calls methods
    def optparser_lsblocked(self):
        usage = "usage: lsblocked [options]\n" \
                "List blocked calls according to the specified options\n" \
                "\n" \
                "Examples:\n" \
                "1) List blocked calls:\n" \
                "     lsblocked\n\n" \
                "2) List blocked PSTN IN calls:\n" \
                "     lsblocked -d IN\n\n" \
                "3) List all PSTN OUT calls:\n" \
                "     lsblocked -d OUT\n\n" \
                "4) List blocked calls started not sooner than \"2010-01-01 00:00:00\" and ended not later than \"2010-01-02 00:00:00\":\n" \
                "     lsblocked -s \"2010-01-01 00:00:00\" -e \"2010-01-02 00:00:00\"\n\n"
        return self.basic_optparser(usage)

    def help_lsblocked(self):
        self.optparser_lsblocked().print_help()
        self.stdout.write( "\n" )
        return False    

    def do_lsblocked(self, line):
        try:
            (options, positional) = self.parseCmd( line,
                                                   self.optparser_lsblocked() )
            
            session = self.m_callDataRegistry.getNewDBSession()
            allCallsQuery = self.m_callDataRegistry.getBlockedCallsQuery(session)
            callsWithDirectionQuery = self. \
                                      getCallsWithDirectionFromQuery(allCallsQuery,
                                                                     options.direction)
            callsInTimeRangeQuery = self.m_callDataRegistry. \
                                    getCallsInTimeRangeFromQuery( callsWithDirectionQuery,
                                                                  options.startTime,
                                                                  options.endTime)
            if options.maxNumCalls is None:
                printCallDataList( callsInTimeRangeQuery.all(), self.stdout )
            else:
                options.maxNumCalls = -1 * options.maxNumCalls
                printCallDataList( callsInTimeRangeQuery. \
                                   order_by(CallData.m_startTimestamp)[options.maxNumCalls:],
                                   self.stdout )
            
        except ValueError, e:
            log_error( fileScopeLogger,
                       "failed to parse command options \"%s\"",
                       line )
            
        return False

    # lsconnected calls methods
    def optparser_lsconnected(self):
        usage = "usage: lsconnected [options]\n" \
                "List connected calls according to the specified options\n" \
                "\n" \
                "Examples:\n" \
                "1) List connected calls:\n" \
                "     lsconnected\n\n" \
                "2) List connected PSTN IN calls:\n" \
                "     lsconnected -d IN\n\n" \
                "3) List all PSTN OUT calls:\n" \
                "     lsconnected -d OUT\n\n" \
                "4) List connected calls started not sooner than \"2010-01-01 00:00:00\" and ended not later than \"2010-01-02 00:00:00\":\n" \
                "     lsconnected -s \"2010-01-01 00:00:00\" -e \"2010-01-02 00:00:00\"\n\n"
        return self.basic_optparser(usage)

    def help_lsconnected(self):
        self.optparser_lsconnected().print_help()
        self.stdout.write( "\n" )
        return False    

    def do_lsconnected(self, line):
        try:
            (options, positional) = self.parseCmd( line,
                                                   self.optparser_lsconnected() )
            
            session = self.m_callDataRegistry.getNewDBSession()
            allCallsQuery = self.m_callDataRegistry.getConnectedCallsQuery(session)
            callsWithDirectionQuery = self. \
                                      getCallsWithDirectionFromQuery(allCallsQuery,
                                                                     options.direction)
            callsInTimeRangeQuery = self.m_callDataRegistry. \
                                    getCallsInTimeRangeFromQuery( callsWithDirectionQuery,
                                                                  options.startTime,
                                                                  options.endTime)
            if options.maxNumCalls is None:
                printCallDataList( callsInTimeRangeQuery.all(), self.stdout )
            else:
                options.maxNumCalls = -1 * options.maxNumCalls
                printCallDataList( callsInTimeRangeQuery. \
                                   order_by(CallData.m_startTimestamp)[options.maxNumCalls:],
                                   self.stdout )
            
        except ValueError, e:
            log_error( fileScopeLogger,
                       "failed to parse command options \"%s\"",
                       line )
            
        return False

    # lsconnected calls methods
    def optparser_rm(self):
        usage = "usage: rm [options]\n" \
                "Remove all inactive calls accorindg to the specified options\n" \
                "\n" \
                "Examples:\n" \
                "1) Remove all inactive calls:\n" \
                "     rm\n\n" \
                "2) Remove PSTN IN inactive calls:\n" \
                "     rm -d IN\n\n" \
                "3) Remove PSTN OUT inactive calls:\n" \
                "     rm -d OUT\n\n" \
                "4) Remove inactive calls ended not later than \"2010-01-02 00:00:00\":\n" \
                "     rm -e \"2010-01-02 00:00:00\"\n\n"
        return self.basic_optparser(usage)

    def help_rm(self):
        self.optparser_rm().print_help()
        self.stdout.write( "\n" )
        return False
    
    def do_rm(self, line):
        self.stdout.write("removing inactive call data...")
        try:
            (options, positional) = self.parseCmd( line,
                                                   self.optparser_lsconnected() )
            
            session = self.m_callDataRegistry.getNewDBSession()
            allCallsQuery = self.m_callDataRegistry.getInactiveCallsQuery(session)
            callsWithDirectionQuery = self. \
                                      getCallsWithDirectionFromQuery(allCallsQuery,
                                                                     options.direction)
            callsInTimeRangeQuery = self.m_callDataRegistry. \
                                    getCallsInTimeRangeFromQuery( callsWithDirectionQuery,
                                                                  options.startTime,
                                                                  options.endTime)
            calls = None
            if options.maxNumCalls is None:
                calls = callsInTimeRangeQuery.all()
            else:
                options.maxNumCalls = -1 * options.maxNumCalls
                calls = callsInTimeRangeQuery. \
                        order_by(CallData.m_startTimestamp)[options.maxNumCalls:]                
            for call in calls:
                session.delete(call)
            session.commit()
            self.stdout.write( "removed inactive calls\n\n" )
            
        except dbexcep.DBAPIError, e:
            log_error(fileScopeLogger, "failed to remove inactive calls\n" )
            session.rollback()
            
        except ValueError, e:
            log_error( fileScopeLogger,
                       "failed to parse command options \"%s\"",
                       line )
            
        return False

    def help_stats(self):
        self.stdout.write( "usage stats\n" )
        self.stdout.write( "Show calls statistics for the last 1 hour\n" )
        return False
    
    def do_stats(self, line):
        intervalStart = datetime.datetime.now()-\
                        datetime.timedelta(seconds=3600)
        stats = self.m_callDataRegistry. \
                getIntervalStats(str(intervalStart),3600)
        self.stdout.write( "Calls statistics for the interval started " \
                           "at %s\n\n" % \
                           intervalStart )
        self.stdout.write( "%30s   %10s %10s %10s\n" % \
                           ("", "PSTN IN", "PSTN OUT", "TOTAL" ) )
        self.stdout.write( "%30s : %10d %10d\n" % \
                           ("Number of calls", \
                            stats["gwNumInboundCalls"], \
                            stats["gwNumOutboundCalls"] ) )
        self.stdout.write( "%30s : %10d %10d\n" % \
                           ("Min number of active calls", \
                            stats["gwMinConcurrentInboundCalls"], \
                            stats["gwMinConcurrentOutboundCalls"] ) )
        self.stdout.write( "%30s : %10d %10d %10d\n" % \
                           ("Max number of active calls", \
                            stats["gwMaxConcurrentInboundCalls"], \
                            stats["gwMaxConcurrentOutboundCalls"], \
                             stats["gwMaxConcurrentCalls"] ) )
        self.stdout.write( "\n%30s\n" % \
                           ("Blocked calls" ) )
        self.stdout.write( "%30s : %10d %10d\n" % \
                           ("Busy calls", \
                            stats["gwNumBusyBlockedInboundCalls"], \
                            stats["gwNumBusyBlockedOutboundCalls"] ) )
        self.stdout.write( "%30s : %10d %10d\n" % \
                           ("No route found", \
                            stats["gwNumNotFoundBlockedInboundCalls"], \
                            stats["gwNumNotFoundBlockedOutboundCalls"] ) )
        self.stdout.write( "%30s : %10d %10d\n" % \
                           ("Other type of failure(s)", \
                            stats["gwNumNotFoundBlockedInboundCalls"],
                            stats["gwNumNotFoundBlockedOutboundCalls"] ) )
        self.stdout.write( "\n%30s\n" % \
                           ("Number of calls with duration" ) )
        self.stdout.write( "%30s : %10d %10d\n" % \
                           ("<=6 seconds",
                            stats["gwNumLessThan6sInboundCalls"],
                            stats["gwNumLessThan6sOutboundCalls"] ) )
        self.stdout.write( "%30s : %10d %10d\n" % \
                           ("<=15 seconds", \
                            stats["gwNumLessThan15sInboundCalls"], \
                            stats["gwNumLessThan15sOutboundCalls"] ) )
        self.stdout.write( "%30s : %10d %10d\n" % \
                           ("<=30 seconds", \
                            stats["gwNumLessThan30sInboundCalls"], \
                            stats["gwNumLessThan30sOutboundCalls"] ) )
        self.stdout.write( "%30s : %10d %10d\n" % \
                           ("<=60 seconds",
                            stats["gwNumLessThan30sInboundCalls"],
                            stats["gwNumLessThan30sOutboundCalls"] ) )
        return False

def printCallData(in_callData, in_stdout=sys.stdout):

    # -----------------
    # time: start call 'callID'
    # time: previousState -- event --> newState
    # parameter1
    # parameter 2
    # ...
    # time: end call 'callID'
    in_stdout.write( "%s : start call '%s' '%s'\n" %
                     ( in_callData.m_startTimestamp,in_callData.m_callId,
                       in_callData.m_direction ) )
    strState = "IDLE"
    for event in in_callData.m_events:
        in_stdout.write( "%s : %s --- %s ---> %s" %
                         (event.m_timestamp, strState,
                          event.m_type, event.m_newState ))
        strState = event.m_newState
        for param in event.m_callParameters:
                in_stdout.write( "\n%s" % param )
                
        in_stdout.write( "\n" )

    if in_callData.m_endTimestamp is not None:
        callDuration = in_callData.m_endTimestamp - in_callData.m_startTimestamp 
        in_stdout.write( "%s : end call '%s' - call duration %s\n" %
                         ( in_callData.m_endTimestamp,in_callData.m_callId,
                           callDuration) )

def printCallDataList( in_callDataList, in_stdout=sys.stdout):
    uiNumCalls = 0
    for call in in_callDataList:
        in_stdout.write( "--------------------------------------------------\n" )
        printCallData( call, in_stdout )
        uiNumCalls += 1

    in_stdout.write( "--------------------------------------------------\n" )
    in_stdout.write( "\nFound %s calls\n" % uiNumCalls )
    in_stdout.write( "\n" )
    
    
if __name__ == '__main__': 

    try:
        #fileScopeLogger.setLogLevel(LogLevel.debug)

        # Parse command line
        textWrapper = TextWrapper()
        usage = "usage: %prog [options] call-data-db-url\n\n" + \
                "\n".join(
                textWrapper.wrap( \
                "The gateway maintains in a database (specified by the call-data-db-url parameter) the a table of all calls. This table contains active (calls that are still live on the gateway) and inactive calls (terminated calls).  For each call, the gateway keeps in the database some important information such as: start time, end time, current state and list of the important events during the life cycle of the call. It is important to note that, the call database does not intend to replace the gateway log file.  It contains a subset of all events that happen on call.  You shall use the gateway log file to retrieve all details about a call.") ) + "\n\n" + \
                "Object model of the Call Data database:\n\n" \
                "+------------------+\n" \
                "| CallDataRegistry |\n" \
                "+------------------+\n" \
                "         |1\n" \
                "         |\n" \
                "         |* calls\n" \
                "    +----------+    events  +-----------+   parameters +---------------+\n" \
                "    | CallData | ---------- | CallEvent | ------------ | CallParameter |\n" \
                "    +----------+ 1        * +-----------+ 1         *  +---------------+\n\n" \
                "\n" \
                "Call state diagram:\n\n" \
                "                                                connect\n" \
                "                                                early-reject (assist xfer)\n" \
                "                                                routing-failure (assist xfer)\n" \
                "                                                outleg-reject (assist xfer)\n" \
                "                                               +------------------+\n" \
                "                                               |                  |\n" \
                "       setup           connect                 v transfer         |\n" \
                "IDLE --------> SETUP -----------> CONNECTED ---------------> TRANSFER_SETUP\n" \
                "                 |                      |                         |\n" \
                "                 |                      | disconnect              |\n" \
                "                 |                      |                         |\n" \
                "                 |                      v                         |\n" \
                "                 +-------------> DISCONNECTED* <------------------+\n" \
                "                 |  disconnect                  disconnect\n" \
                "                 |                              early-reject (blind xfer)\n" \
                "                 |                              routing-failure (blind xfer)\n"\
                "                 |                              outleg-reject (blind xref)\n" \
                "                 |\n" \
                "                 +----------------------> BLOCKED* \n" \
                "                       early-reject\n" \
                "                       routing-failure\n" \
                "                       outleg-reject\n" \
                "\n" \
                "\n" \
                " ANY-STATES ---+\n" \
                "      ^        |\n" \
                "      |        |\n" \
                "      +--------+\n" \
                "         info\n" \
                "\n" + \
                "\n".join( textWrapper.wrap( \
                "The main goal of this application is to provide a mean to interrogate this database to identify calls that are meeting some criterias (example: all blocked call, all active calls...) For each calls, the application provides a high level summary of the call.\n" )) + \
                "\n\n" + \
                "\n".join( textWrapper.wrap( "call-data-db-url specific the location of the call data database.  This value shall matches the value of the parameter \"netborder.callData.db.url\n if you want to work with the database currently used by the gateway.")) + \
                "\n".join( textWrapper.wrap( "The usual URL for that gateway call database is:" )) + \
                "\n\n" \
                "  sqlite:///${netborder.Installation.Directory}/logs/db/CallData.db\"" + \
                "\n\n\n" + \
                "Examples:\n\n" + \
                "1) Start the gateway call data viewer in interractive mode:\n" \
                "   netborder-python GWCallDataViewer.py \"sqlite:///c:/Program Files/Netborder/Express/Gateway/logs/db/CallData.db\"\n\n" \
                "2) Start the gateway call data viewer to execute one command:\n" \
                "   netborder-python GWCallDataViewer.py \"sqlite:///c:/Program Files/Netborder/Express/Gateway/logs/db/CallData.db -c \"ls\"\n\n" + \
                "3) Obtain a list of all commands supported by the viewer:\n" \
                "   netborder-python GWCallDataViewer.py \"sqlite:///c:/Program Files/Netborder/Express/Gateway/logs/db/CallData.db -c \"help\"\n\n" + \
                "\n" \
                "Return values:\n\n" \
                "  0 in value of success\n" \
                "  none 0 value in case of error\n"
        
        parser = OptionParser(usage=usage)
        parser.add_option( "-c", "--command", type="string",
                           dest="strCommandsToExecute",
                           help= "A string that is a semicolon delimited list of editor commands.  When this option is specified, the application runs the specified commands and returns immediately. When this option is not specified, the tool enters interactive mode. The interactive mode can be used by users who wish to pipe their commands (one per line) to the tool's standard input.")

        (options, callDataDBUrl) = parser.parse_args()

        if len(callDataDBUrl) == 0:
            log_error( fileScopeLogger, "missing call data database URL")
            sys.exit(1)

        strCallDataDBUrl = callDataDBUrl[0]

        # Load call data DB
        log_debug( fileScopeLogger,
                   "open CallData DB at URL: %s", strCallDataDBUrl )
        callDataRegistry = GWCallDataRegistry()
        if callDataRegistry.configure(strCallDataDBUrl, False, False ) == False:
            log_error( fileScopeLogger,
                       "failed to load parameter DB from file: %s",
                        strCallDataDBUrl )
            sys.exit(2)

        # Initalize command interpreter
        if options.strCommandsToExecute is None:
            log_debug( fileScopeLogger, "execute commands from the stdin" )
            callDataViewer = GWCallDataViewer(in_callDataRegistry=callDataRegistry)
        else:
            log_debug( fileScopeLogger,
                       "execute command specified as an command " \
                       "line argument: %s",
                       options.strCommandsToExecute )

            # Store command to execute in tempfile
            options.strCommandsToExecute = options.strCommandsToExecute.replace( ";", "\n" )
            commandStream = tempfile.TemporaryFile("r+" )
            commandStream.write( options.strCommandsToExecute + "\n" )
            commandStream.seek( 0, os.SEEK_SET )
            callDataViewer = GWCallDataViewer(in_callDataRegistry=callDataRegistry,
                                              stdin=commandStream,
                                              noprompt=True )

        #If you want a given stdin to be used, make sure to set the
        #instance use_rawinput attribute to False, otherwise stdin
        #will be ignored.
        callDataViewer.use_rawinput = False

        # Run command interpreter
        callDataViewer.cmdloop()

        # Success

    except SystemExit, e:
        log_debug( fileScopeLogger, "" )
## !!MIRROR FILE, DO NOT MODIFY!!
