# Author: Jonathan Maurice
# Date:   February 11 2011
# 
# Description: This file contain a Job interface which all job can inherit from.
#              A JobRunner class is also defined, this is used to run a group
#              of job in the order they are added.
#
#
# Copyright (C) 2011 Paraxip Technologies
# All Rights Reserved.
#
# This code is Paraxip Technologies Confidential Property.
# Use of and Access to this code is covered by a previously executed
# Non-disclosure Agreement between Paraxip Technologies and the Recipient.
# This code is being supplied for evaluation purposes only and is not to be
# used for any other purpose.  
#

from paraxip.infra.PyLoggingFunctions import *
from paraxip.infra.Assert import *
fileScopeLogger = Logger("netborder.infra.PyJob")


class Job(object):
  """Job interface, this class only has a run method"""

  def __init__(self):
    super(Job, self).__init__()
  
  def run(self):
    """Job method"""
    pass



class JobRunner(Job):
  """Job container used to run a group of jobs and notify
     observer on progress"""
  def __init__(self, jobStatus=None):
    super(JobRunner, self).__init__()
    self._jobStatus = jobStatus
    self._jobList = list()
    self._observerList = list()
    

  def configure(self):
    """Configure the job runner"""
    pass


  def addJob(self, job):
    """Add a job to the container"""
    logAssert(fileScopeLogger, hasattr(job, "run"))
    self._jobList.append(job)


  def addObserver( self, observer):
    """Add a job progress observer"""
    logAssert(fileScopeLogger, hasattr(job, "onProgress"))
    self._observerList.append(observer)


  def run(self):
    """Call run method on each job inside the container"""
    
    for job in self._jobList:

      log_debug(fileScopeLogger, "Running job %s (id: %s)" % 
                (type(job), id(job)))
      result = job.run()
      
      if result is False:
        log_debug(fileScopeLogger, "Job returned False %s (id: %s)" % 
                  (type(job), id(job)))
        return False 

      for observer in self._observerList:
        log_debug(fileScopeLogger, "Notifying observer %s (id: %s)" % 
                  (type(observer), id(observer)))
        observer.onProgress()

    return True


class JobStatus(object):
  """Monitor status of the job being run"""

  def __init__(self):
    super(JobStatus, self).__init__()
    self.flags = {'warning':0, 'error':0}
    self.messages = {'warning': list(), 'error': list()}

  def addWarning(self, msg=None):
    """Increment warning flag and store message if any"""

    log_debug(fileScopeLogger, "Incrementing warning flags to %d" % 
              (self.flags['warning']+1))
    self.flags['warning'] += 1

    if msg is not None:
      log_debug(fileScopeLogger, "Adding warning message '%s'" % msg)
      self.messages['warning'].append(msg)

    return True

  def addError(self, msg=None):
    """Increment error flag and store message if any"""
    
    log_debug(fileScopeLogger, "Incrementing error flags to %d" %
              (self.flags['error']+1))
    self.flags['error'] += 1

    if msg is not None:
      log_debug(fileScopeLogger, "Adding error message '%s'" % msg)
      self.messages['error'].append(msg)

    return True

  def getWarning(self):
    """Return the warning message list"""
    return self.messages['warning']

  def getError(self):
    """Return the error message list"""
    return self.messages['error']

  def getWarningNumber(self):
    """Return the number of warning"""
    return self.flags['warning']

  def getErrorNumber(self):
    """Return the number of error"""
    return self.flags['error']

  def hasWarning(self):
    """Return True if there are any warning"""
    return True if self.flags['warning'] > 0 else False

  def hasError(self):
    """Return True if there are any error"""
    return True if self.flags['error'] > 0 else False
    


###############################################################################
# Runtest
if __name__ == '__main__':

  fileScopeLogger.setLogLevel(LogLevel.info)

  def runtest():
    """Create simple Job and add them to the Job runner"""
    
    log_info(fileScopeLogger, "Starting the runtest")
    
    jobIdList = list()
    class JobId(Job):
      
      def __init__(self, jobId):
        super(JobId, self).__init__()
        self.jobId = jobId
        
      def run(self):
        """Add job Id to list"""
        jobIdList.append(self.jobId)


    myIdList = ["Job 1", "Job 2", "Job 3", "Job 4", "Job 5"]
    myJobRunner = JobRunner()
    for jId in myIdList:
      log_info(fileScopeLogger, "Adding job with Id '%s'" % jId)
      myJobRunner.addJob(JobId(jId))
    
    myJobRunner.run()
    
    if len(myIdList) == len(jobIdList):
      for i in range(len(myIdList)):
        
        if myIdList[i] == jobIdList[i]:
          continue
        else:
          log_error(fileScopeLogger, 
                   "Job Id '%s' shoud be equal to '%s'" % (jobIdList[i],
                                                           myIdList[i]))
    
    else:
      log_error(fileScopeLogger, 
                "JobRunner container lenght '%u' is not the same as the " 
                % len(jobIdList) + "number of added job '%u'" % len(myIdList) + 
                ". This probably mean that some jobs fail to be added.")


  runtest()
## !!MIRROR FILE, DO NOT MODIFY!!
