"""

File:    PbxConfigManager.py
Author:  Jonathan Maurice <jmaurice@sangoma.com>
Date:    4/5/2011 4:50:28 PM
 
Description: Manager of the pbx-configurator process. This class is responsible
             of creating the working directory of the application in order to 
             create the configuration file from the pbx xml file. This class
             also as useful accessors, which are used by most of the other
             class.
 
Copyright (C) 2003-2011 Sangoma Technologies. 
All Rights Reserved.

This code is Sangoma Technologies Confidential Property.
Use of and Access to this code is covered by a previously executed
Non-disclosure Agreement between Sangoma Technologies and the Recipient.
This code is being supplied for evaluation purposes only and is not to be
used for any other purpose.  

"""

import os
import shutil

from paraxip.infra.PyLoggingFunctions import *
from netborderPyInfra import TraceScope
from netborderPyInfra import PyGlobalConfig as globalConfig

from paraxip.smart_update.TempDirManager import TempDirManager
from paraxip.smart_update.XsltProcWrapper import XsltProcWrapper


fsLogger = Logger("netborder.pbxConfigurator.PbxConfigManager")

class PbxConfigManager(TempDirManager):
  """Manage the pbx-configurator process for creating gateway config files"""

  def __init__(self, pbxFilePath, pbxConfigInfo, finalConfigDir, 
               replaceStrDict=None, replaceFileExtension=None):
    t = TraceScope(fsLogger, "PbxConfigManager.__init__")

    self._workingDir = \
      globalConfig.getParameter('netborder.pbxConfigurator.Working.Directory')
    if self._workingDir is None:
       raise Exception("netborder.pbxConfigurator.Working.Directory as not "\
                       "been set.")

    self._installDir = \
      globalConfig.getParameter('netborder.Installation.Directory')
    if self._installDir is None:
      raise Exception("netborder.Installation.Directory as not "\
                      "been set.")

    self._binDir = \
      globalConfig.getParameter('netborder.Installation.Bin.Directory')
    if self._binDir is None:
      raise Exception ("netborder.Installation.Bin.Directory as not "\
                       "been set.")

    pbxFileDirPath = os.path.join(self._workingDir, 'pbxFile')
    super(PbxConfigManager, self).__init__(pbxFileDirPath, finalConfigDir, 
                                           replaceStrDict, replaceFileExtension)

    self._originalPbxFilePath = pbxFilePath
    self._pbxFileDirPath = pbxFileDirPath
    pbxFileName = os.path.basename(pbxFilePath)
    self._pbxFilePath = os.path.join(self._pbxFileDirPath, pbxFileName)
    self._pbxConfigInfo = pbxConfigInfo

    self._jobAdderFileName = 'PbxConfigJobAdder.py'
    self._xsltProc = None


  def initialize(self):
    """Make sure working directory is created an empty and set mode 
       to Initial"""
    t = TraceScope(fsLogger, "PbxConfigManager.initialize")
    
    self._tmpDir1 = os.path.join(self._workingDir, 'tmpCfg1')
    self._tmpDir2 = os.path.join(self._workingDir, 'tmpCfg2')

    if self.cleanTempDirectories() is False:
      return False

    #Create initial directory (used to store the pbx file)
    if os.path.exists(self._pbxFileDirPath):
      log_debug(fsLogger, "Deleting initial directory '%s'" %
                self._pbxFileDirPath)
      try:
        shutil.rmtree(self._pbxFileDirPath)
      except (OSError, IOError), e:
        log_error(fsLogger, "Unable to remove directory '%s'. Error=%s" %
                  (self._pbxFileDirPath, e))
        return False

    log_debug(fsLogger, "Creating initial directory '%s'" %
              self._pbxFileDirPath
    )
    try:
      os.makedirs(self._pbxFileDirPath)
    except OSError, e:
      log_error(fsLogger, "Unable to create directory '%s'. Error=%s" %
                (self._pbxFileDirPath, e))
      return False

    #Make a copy of the provided pbx file.
    log_debug(fsLogger, "Copying file '%s' to '%s'" % 
              (self._originalPbxFilePath, self._pbxFilePath)
    )
    try:
      shutil.copy2(self._originalPbxFilePath, self._pbxFilePath)
    except (OSError, IOError), e:
      log_error(fsLogger, "Unable to copy file '%s' to '%s'. Error=%s"\
                % (self._originalPbxFilePath, self._pbxFilePath, e)) 
      return False

    #Need to replace na_me string with name before generating JobAdder due
    #to a 3CX bug...
    if not self._replaceStringInFiles(self._pbxFileDirPath):
      log_error(fsLogger, "Failed to replace strings in files of directory %s", 
                self._pbxFileDirPath
      )
      return False

    #Configure XsltProc
    xsltScriptDir = \
        globalConfig.getParameter('netborder.pbxConfigurator.XsltScripts.Directory')
    pbxCfgParserXsl = os.path.join(xsltScriptDir, 'pbxConfigParser.xsl')

    if not os.path.exists(pbxCfgParserXsl):
      log_error(fsLogger, "Cannot find file %s" % pbxCfgParserXsl)
      return False

    jobAdderDirPath = os.path.dirname(os.path.abspath(__file__))
    jobAdderFilePath = os.path.join(jobAdderDirPath, self._jobAdderFileName)
    self._xsltProc = XsltProcWrapper(xslFile=pbxCfgParserXsl,
                                     xmlFile=self._pbxFilePath,
                                     outFile=jobAdderFilePath)

    #Generate PbxConfigJobAdder.py
    log_debug(fsLogger, "Generating %s from %s" %
             (self._jobAdderFileName, self._originalPbxFilePath)
    )
    (rc, msg, warningList) = self._xsltProc.call()
    if not rc == 0:
      log_error(
        fsLogger,
        "Failed to parse %s file. rc=%s, cause=%s." % 
        (self._originalPbxFilePath, rc, msg)
      )
      return False

    #_srcDir will be _initCfgDir (_pbxFileDirPath)
    #_dstDir will be _tmpDir1 (_workingDir/tmpCfg1)
    return self.setInitialMode()


  def getInstallDirectory(self):
    """Return installation directory"""
    t = TraceScope(fsLogger, "PbxConfigManager.getInstallDirectory")
    return self._installDir

  def getBinDirectory(self):
    """Return binaries directory"""
    t = TraceScope(fsLogger, "PbxConfigManager.getBinDirectory")
    return self._binDir
  
  def getWorkingDirectory(self):
    """Return the working directory"""
    t = TraceScope(fsLogger, "PbxConfigManager.getWorkingDirectory")
    return self._workingDir

  def getPbxFileName(self):
    """Return the name of the pbx file"""
    t = TraceScope(fsLogger, "PbxConfigManager.getPbxFilePath")
    return os.path.basename(self._pbxFilePath)

  def getPbxFilePath(self):
    """Return path to pbx file"""
    t = TraceScope(fsLogger, "PbxConfigManager.getPbxFilePath")
    return self._pbxFilePath

  def getAppVersion(self):
    """Return version of the application being configured"""
    t = TraceScope(fsLogger, "PbxConfigManager.getAppVersion")
    return self._pbxConfigInfo.getAppVersion()

  def getAppName(self):
    """Return name of the application being configured"""
    t = TraceScope(fsLogger, "PbxConfigManager.getAppName")
    return self._pbxConfigInfo.getAppName()

  def getOemName(self):
    """Return oem name of the provided pbx file"""
    t = TraceScope(fsLogger, "PbxConfigManager.getOemName")
    return self._pbxConfigInfo.getOemName()

  def getOemVersion(self):
    """Return oem version of the provided pbx file"""
    t = TraceScope(fsLogger, "PbxConfigManager.getOemVersion")
    return self._pbxConfigInfo.getOemVersion()



###############################################################################
# Runtest
if __name__ == '__main__':

  testFail = False

  Logger('netborder.smartUpdate').setLogLevel(LogLevel.debug)
  fsLogger.setLogLevel(LogLevel.debug)

  pbxFile = os.path.join('test-input', '3cx-a200-2fxo-2fxs.pbx')
  cfgDir = os.path.join('test-output', 'pbxConfigManager-test')
  replaceStrDict = {'na_me':'name'}

  #This is the same information as in the pbx file 3cx-a200-2fxo-2fxs.pbx
  configInfo = {'version':'1.0',
                'applicationName':'Netborder Express Gateway',
                'applicationVersion':'4.1.1',
                'oemName':'3CX',
                'oemVersion':'9.0'}

  from paraxip.pbx_configurator.PbxConfigurationInfo import PbxConfigurationInfo
  pbxCfgInfo = PbxConfigurationInfo(pbxInfoDict=configInfo)
  pbxCfgMan = PbxConfigManager(pbxFile, pbxCfgInfo, cfgDir, 
                               replaceStrDict, '.pbx')

  if not pbxCfgMan.initialize():
    log_error(fsLogger, "Failed to initialize PbxConfigManager")
    testFail = True

  if not pbxCfgMan.initialConvert():
    log_error(fsLogger, "PbxConfigManager initial convert failed")
    testFail = True

  #Pbx file should be copied.
  if not os.path.isfile(pbxCfgMan._pbxFilePath):
    log_error(fsLogger, "PbxConfigManager failed to copy the pbx file %s" %
              pbxCfgMan._pbxFilePath)
    testFail = True

  #JobAdder should be created.
  if not os.path.exists(pbxCfgMan._jobAdderFileName):
    log_error(fsLogger, "PbxConfigManager failed to generate %s file" %
              pbxCfgMan._jobAdderFileName)
    testFail = True
  else:
    os.remove(pbxCfgMan._jobAdderFileName)

  #Test accessors.
  pbxFilePathCopy = \
      os.path.join(pbxCfgMan._pbxFileDirPath, os.path.basename(pbxFile))

  if not pbxCfgMan.getPbxFilePath() == pbxFilePathCopy:
    log_error(fsLogger, "PbxConfigManager pbx file copy path '%s' is not '%s'"%
             (pbxCfgMan.getPbxFilePath(), pbxFilePathCopy)
    )
    testFail = True

  if not pbxCfgMan.getAppVersion() == configInfo['applicationVersion']:
    log_error(fsLogger, "PbxConfigManager application version '%s' is not '%s'"%
             (pbxCfgMan.getAppVersion(), configInfo['applicationVersion'])
    )
    testFail = True

  if not pbxCfgMan.getAppName() == configInfo['applicationName']:
    log_error(fsLogger, "PbxConfigManager application name '%s' is not '%s'"%
             (pbxCfgMan.getAppName(), configInfo['applicationName'])
    )
    testFail = True

  if not pbxCfgMan.getOemName() == configInfo['oemName']:
    log_error(fsLogger, "PbxConfigManager oem name '%s' is not '%s'"%
             (pbxCfgMan.getOemName(), configInfo['oemName'])
    )
    testFail = True

  if not pbxCfgMan.getOemVersion() == configInfo['oemVersion']:
    log_error(fsLogger, "PbxConfigManager oem version '%s' is not '%s'"%
             (pbxCfgMan.getOemVersion(), configInfo['oemVersion'])
    )
    testFail = True

  if not pbxCfgMan.setTempMode():
    log_error(fsLogger, "Failed to set PbxConfigManager to temp mode")
    testFail = True

  #Verify that directories were created.
  if not os.path.isdir(pbxCfgMan.getSrcDir()):
    log_error(fsLogger, "Directory %s was not created" % 
             pbxCfgMan.getSrcDir()
    )
    testFail = True

  if not os.path.isdir(pbxCfgMan.getDstDir()):
    log_error(fsLogger, "Directory %s was not created" % 
             pbxCfgMan.getDstDir()
    )
    testFail = True

  if testFail:
    log_error(fsLogger, "Runtest failed!")
    print("Failed")
  else:
    log_info(fsLogger, "Runtest passed!")
    print("Passed")
   




## !!MIRROR FILE, DO NOT MODIFY!!
