"""

File:    PbxConverter.py
Author:  Jonathan Maurice <jmaurice@sangoma.com>
Date:    4/7/2011 3:10:55 PM
 
Description: Contain a set of converters that are used to generate specific
gateway configuration file.
 
Copyright (C) 2003-2011 Sangoma Technologies. 
All Rights Reserved.

This code is Sangoma Technologies Confidential Property.
Use of and Access to this code is covered by a previously executed
Non-disclosure Agreement between Sangoma Technologies and the Recipient.
This code is being supplied for evaluation purposes only and is not to be
used for any other purpose.  

"""

import os
import shutil
import urllib

from paraxip.infra.PyLoggingFunctions import  *
from netborderPyInfra import TraceScope
from netborderPyInfra import PyGlobalConfig as globalConfig
from paraxip.smart_update.Converter import Converter
from paraxip.smart_update.XsltProcWrapper import XsltProcWrapper
import netborderPySangoma


fsLogger = Logger("netborder.pbxConfigurator.PbxConverter")

nullConvLogger = \
    Logger("netborder.pbxConfigurator.PbxConverter.NullConverter")
extractCfgLogger = \
    Logger("netborder.pbxConfigurator.PbxConverter.ExtractConfigData")
copyCfgLogger = \
    Logger("netborder.pbxConfigurator.PbxConverter.CopyConfig")
wanpipeSetterLogger = \
    Logger("netborder.pbxConfigurator.PbxConverter.WanpipeSetter")


class NullConverter(Converter):
  """Empty converter"""

  def __init__(self, *args, **kwargs):
    t = TraceScope(nullConvLogger, "NullConverter.__init__")
    super(NullConverter, self).__init__(*args, **kwargs)

  def run(self):
    """Always return True"""
    t = TraceScope(nullConvLogger, "NullConverter.run")
    return True
    

class PbxConverter(Converter):
  """Pbx converters base class"""

  def __init__(self, pbxConfigMan, group, inputFilename, outputFilename,
               jobStatus=None, xsltScript=None, inputValidationFile=None):
    t = TraceScope(fsLogger, "PbxConverter.__init__")
    super(PbxConverter, self).__init__(pbxConfigMan, group, inputFilename,
                                      outputFilename, jobStatus=jobStatus,
                                      xsltScript=xsltScript,
                                      inputValidationFile=inputValidationFile)

    self.conversionFilesDir = \
        globalConfig.getParameter("netborder.pbxConfigurator.XsltScripts.Directory")

    self.xsltProc = XsltProcWrapper()

  def run(self):
    """Abstract method"""
    t = TraceScope(fsLogger, "PbxConverter.run")
    raise NotImplementedError("PbxConverter.run should not be called."\
                              "Abstract class!")
    return False
    

class ExtractConfigData(PbxConverter):
  """Extract all data childs element of a job from the pbx xml file.
     If the outputFilename has a .properties extension it's converted
     to the properties file format."""

  def __init__(self, pbxConfigMan, group, inputFilename, outputFilename,
               jobStatus=None, xsltScript=None, inputValidationFile=None):
    t = TraceScope(extractCfgLogger, "ExtractConfigData.__init__")
    super(ExtractConfigData, self).__init__(pbxConfigMan, group, inputFilename, 
                                            outputFilename, jobStatus, 
                                            xsltScript, inputValidationFile)

    self.extractCfgXslFilePath = os.path.join(self.conversionFilesDir,
                                             'extractConfigFileData.xsl')
    
  def run(self):
    t = TraceScope(extractCfgLogger, "ExtractConfigData.run")

    inputPbxFilePath = os.path.join(self.tempDirManager.getSrcDir(),
                                    self.inputFilename)
    outputFilePath = os.path.join(self.tempDirManager.getDstDir(),
                                  self.outputFilename)
    outFileExt = os.path.splitext(self.outputFilename)[1]

    log_info(extractCfgLogger, "Creating file %s" % self.outputFilename)
    strParam = '--stringparam fileName %s' % self.outputFilename
    self.xsltProc.configure(self.extractCfgXslFilePath, inputPbxFilePath,
                           outFile=outputFilePath,
                           xsltProcOptions=strParam)

    log_debug(extractCfgLogger, 
              "Extracting %s data in config file %s to file %s" %
             (self.outputFilename, inputPbxFilePath, outputFilePath)
    )
    (rc, msg, warningList) = self.xsltProc.call()
    if not rc == 0:
      log_error(
        extractCfgLogger,
        "Failed to extract %s data in file %s. rc=%s, cause=%s." % 
        (self.outputFilename, inputPbxFilePath, rc, msg)
      )
      return False

    #In case of a properties file, it need to be save in the properties format.
    if outFileExt == '.properties':

      log_debug(extractCfgLogger, "Converting data to properties file format")
      outXmlPropFile = os.path.join(self.tempDirManager.getDstDir(),
                                    self.outputFilename + '-xml')
      try:
        log_debug(extractCfgLogger, "Moving file %s to %s" % 
                  (outputFilePath, outXmlPropFile)
        )
        shutil.move(outputFilePath, outXmlPropFile)
      except (OSError, IOError), e:
        log_error(extractCfgLogger, "Unable to move file %s to %s. Error=%s" %
                 (outputFilePath, outXmlPropFile, e)
        )

      xmlToPropScript = os.path.join(self.conversionFilesDir,
                                    "xmlToProperties.xsl")

      self.xsltProc.configure(xmlToPropScript, outXmlPropFile,
                              outFile=outputFilePath)

      log_debug(extractCfgLogger, 
                "Converting file %s to properties format %s" %
              (outXmlPropFile, outputFilePath)
      )
      (rc, msg, warningList) = self.xsltProc.call()
      if not rc == 0:
        log_error(
          extractCfgLogger,
          "Failed to convert file %s to properties format. rc=%s, cause=%s." % 
          (outXmlPropFile, rc, msg)
        )
        return False

      #Remove xml properties file
      log_debug(extractCfgLogger, "Deleting xml properties file %s" % 
                outXmlPropFile
      )
      try:
        os.remove(outXmlPropFile)
      except OSError, e:
        log_warning(extractCfgLogger,
                    "Failed to delete file %s. Cause=%s" %
                    (outXmlPropFile,e)
        )

    return True


class CopyConfig(PbxConverter):
  """Copy a file to a new location"""

  def __init__(self, pbxConfigMan, group, inputFilename, outputFilename,
               jobStatus=None, xsltScript=None, inputValidationFile=None):
    t = TraceScope(copyCfgLogger, "CopyConfig.__init__")
    super(CopyConfig, self).__init__(pbxConfigMan, group, inputFilename, 
                                     outputFilename, jobStatus, 
                                     xsltScript, inputValidationFile)

  def run(self):
    t = TraceScope(copyCfgLogger, "CopyConfig.run")
    inputFilePath = os.path.join(self.tempDirManager.getSrcDir(),
                                 self.inputFilename)
    outputDirPath = self.tempDirManager.getDstDir()
    outputFilePath = os.path.join(outputDirPath, self.outputFilename)

    log_info(copyCfgLogger, "Copying file %s to %s" %
            (self.inputFilename, outputFilePath)
    )
    log_debug(copyCfgLogger, "Copying file from %s to %s" %
             (inputFilePath, outputFilePath)
    )
    return self.copyFile(inputFilePath, outputFilePath)
    

class WanpipeSetter(PbxConverter):
  """Set the wanpipe of a pstn-config file."""
  def __init__(self, pbxConfigMan, group, inputFilename, outputFilename,
               jobStatus=None, xsltScript=None, inputValidationFile=None):
    t = TraceScope(wanpipeSetterLogger, "WanpipeSetter.__init__")
    super(WanpipeSetter, self).__init__(pbxConfigMan, group, inputFilename, 
                                        outputFilename, jobStatus, 
                                        xsltScript, inputValidationFile)

    self.setWanpipeXslFilePath = os.path.join(self.conversionFilesDir,
                                              xsltScript)
    
  def run(self):
    """Save the output of netborder-list-boards in a file and use it to set the
    wanpipe of the pstn-config defined in the pbx config file."""
    t = TraceScope(wanpipeSetterLogger, "WanpipeSetter.run")

    inputPstnCfgPath = os.path.join(self.tempDirManager.getSrcDir(),
                                    self.inputFilename)
    outputPstnCfgPath = os.path.join(self.tempDirManager.getDstDir(),
                                     self.outputFilename)
    listBoardXmlPath = self.tempDirManager.getWorkingDirectory() + \
                       '/listBoards.xml'

    log_info(wanpipeSetterLogger, "Updating wanpipe to match current board "\
             "configuration in %s" % self.outputFilename
    )

    #All \ need to be replaced by /, because the listBoardXmlPath is being
    #included by xsltproc and \ are not supported.
    listBoardXmlPath = listBoardXmlPath.replace('\\','/')

    #Retrieve the board list with netborder-list-board and write it to a file.
    log_debug(wanpipeSetterLogger, "Retrieving board list")
    hwPstnConfig = netborderPySangoma.getBoardsListString()
    log_debug(wanpipeSetterLogger, "Board list is:\n%s" % hwPstnConfig)

    log_debug(wanpipeSetterLogger, "Writting board list xml at %s" %
              listBoardXmlPath
    )
    try:
      listBoardXmlFH = open(listBoardXmlPath, 'w')
    except IOError, e:
      log_error(wanpipeSetterLogger,
                "Failed to open file %s for writting. Error=%s" %
                (listBoardXmlPath, e)
      )
      return False

    listBoardXmlFH.write(hwPstnConfig)
    listBoardXmlFH.close()

    #Xsltproc require path to be in url format (space must be replace by %20).
    strParam = '--stringparam in_boardListFile %s' %\
               urllib.quote(listBoardXmlPath, safe=':/')

    self.xsltProc.configure(self.setWanpipeXslFilePath, inputPstnCfgPath,
                           outFile=outputPstnCfgPath,
                           xsltProcOptions=strParam)

    log_debug(wanpipeSetterLogger, 
              "Setting wanpipe in file %s" % outputPstnCfgPath 
    )
    (rc, msg, warningList) = self.xsltProc.call()
    if not rc == 0:
      log_error(
        wanpipeSetterLogger,
        "Failed to set wanpipe in file %s. rc=%s, cause=%s." % 
        (self.outputFilename, rc, msg)
      )
      return False

    #Delete list board xml file.
    log_debug(wanpipeSetterLogger, 
              "Deleting list board xml file %s" % listBoardXmlPath 
    )
    try:
      os.remove(listBoardXmlPath)
    except OSError, e:
      log_warn(wanpipeSetterLogger,
               "Failed to delete temporary file %s. Error=%s"
               (listBoardXmlPath, e)
      )

    return True

    
###############################################################################
# Runtest
if __name__ == '__main__':
  
  from paraxip.pbx_configurator.PbxConfigManager import PbxConfigManager
  from paraxip.infra.PyJob import JobStatus
  from paraxip.pbx_configurator.PbxConfigurationInfo import PbxConfigurationInfo

  testFail = False
  Logger('netborder.smartUpdate').setLogLevel(LogLevel.debug)
  fsLogger.setLogLevel(LogLevel.debug)

  pbxFile = os.path.join('test-input', '3cx-a200-2fxo-2fxs.pbx')
  cfgDir = os.path.join('test-output', 'pbx-converter-test')
  replaceStrDict = {'na_me':'name'}

  pbxCfgInfo = PbxConfigurationInfo(pbxFilePath=pbxFile)

  testStatus = JobStatus()
  cfgMan = PbxConfigManager(pbxFile, pbxCfgInfo, cfgDir,
                            replaceStrDict, '.pbx')

  if not cfgMan.initialize():
    log_error(fsLogger, "Failed to initialize PbxConfigManager")
    testFail = True

  if not cfgMan.initialConvert():
    log_error(fsLogger, "PbxConfigManager initial convert failed")
    testFail = True

  if not cfgMan.setTempMode():
    log_error(fsLogger, "Failed to set PbxConfigManager to temp mode")
    testFail = True


  #Test ExtractConfigData.
  outputFilePath = os.path.join(cfgMan.getDstDir(), 'gw.properties')
  extCfg = ExtractConfigData(cfgMan, group='gw', 
                            inputFilename=cfgMan.getPbxFileName(),
                            outputFilename='gw.properties',
                            jobStatus=testStatus)

  if not extCfg.run():
    log_error(fsLogger, "Failed to create %s" % extCfg.outputFilename)
    testFail = True

  if not os.path.exists(outputFilePath):
    log_error(fsLogger, "File %s was not created" % outputFilePath)
    testFail = True


  #Test WanpipeSetter.
  outputFilePath = os.path.join(cfgMan.getDstDir(), 'pstn-config.xml')
  extCfg = ExtractConfigData(cfgMan, group='gw', 
                            inputFilename=cfgMan.getPbxFileName(),
                            outputFilename='pstn-config.xml',
                            jobStatus=testStatus)

  if not extCfg.run():
    log_error(fsLogger, "Failed to create %s" % extCfg.outputFilename)
    testFail = True

  if not os.path.exists(outputFilePath):
    log_error(fsLogger, "File %s was not created" % outputFilePath)
    testFail = True

  cfgMan.swapDirectories()
  outputFilePath = os.path.join(cfgMan.getDstDir(), 'pstn-config.xml')
  wanSetter = WanpipeSetter(cfgMan, group='gw', 
                            inputFilename='pstn-config.xml',
                            outputFilename='pstn-config.xml',
                            xsltScript='setWanpipe.xsl',
                            jobStatus=testStatus)

  if not wanSetter.run():
    log_error(fsLogger, "Failed to set wanpipe %s" % wanSetter.outputFilename)
    testFail = True

  if not os.path.exists(outputFilePath):
    log_error(fsLogger, "File %s was not created" % outputFilePath)
    testFail = True


  if testFail:
    log_error(fsLogger, "Runtest failed!")
    print("Failed")
  else:
    log_info(fsLogger, "Runtest passed!")
    print("Passed")
## !!MIRROR FILE, DO NOT MODIFY!!
