#
#  @file   ImportCfg.py
#  @author Jean-Philippe Boily, jr. eng.
#  @date   02/15/2011
#
#  Description:
#
#  Main module of the SmartUpdate import process. Extracts the configuration
#  zip file, validate the configuration to be imported is valid for the current
#  version of the application, and start the process.
#
#  Copyright (C) 2011 Sangoma Technologies.
#  All Rights Reserved.
#
#  This code is Sangoma Technologies Confidential Property.
#  Use of and Access to this code is covered by a previously executed
#  Non-disclosure Agreement between Sangoma Technologies and the Recipient.
#  This code is being supplied for evaluation purposes only and is not to be
#  used for any other purpose.
#
#

__author__="Jean-Philippe Boily, jr. eng."
__date__ ="$15-Feb-2011 4:50:49 PM$"

#-----------------------------------------------------------------------------#
# imports
#-----------------------------------------------------------------------------#
import os
import netborderPyXML
import netborderPyThread
import zipfile
import platform
import urllib
import shutil

from paraxip.infra.PyLoggingFunctions import *
from netborderPyInfra import PyGlobalConfig as globalCfg
from paraxip.smart_update.TempDirManager import *
from paraxip.smart_update.XsltProcWrapper import *
from paraxip.smart_update.SmartUpdateJobRunner import *
from paraxip.infra.PyJob import JobStatus

from xml.dom.minidom import Document

#-----------------------------------------------------------------------------#
# global variables
#-----------------------------------------------------------------------------#
fileScopeLogger = Logger("netborder.smartUpdate.ImportCfg")

def extract(zip, member, path=None, pwd=None):
  """Extract a member from the archive to the current working directory,
     using its full name. Its file information is extracted as accurately
     as possible. `member' may be a filename or a ZipInfo object. You can
     specify a different directory using `path'.
  """
  if not isinstance(member, zipfile.ZipInfo):
    member = zip.getinfo(member)

  if path is None:
    path = os.getcwd()

  return extract_member(zip, member, path, pwd)

def extractall(zip, path=None, members=None, pwd=None):
  """Extract all members from the archive to the current working
     directory. `path' specifies a different directory to extract to.
     `members' is optional and must be a subset of the list returned
     by namelist().
  """
  if members is None:
    members = zip.namelist()

  for zipinfo in members:
    extract(zip, zipinfo, path, pwd)

  return True

def extract_member(zip, member, targetpath, pwd):
  """Extract the ZipInfo object 'member' to a physical
     file on the path targetpath.
  """
  # build the destination pathname, replacing
  # forward slashes to platform specific separators.
  # Strip trailing path separator, unless it represents the root.
  if (targetpath[-1:] in (os.path.sep, os.path.altsep)
      and len(os.path.splitdrive(targetpath)[1]) > 1):
    targetpath = targetpath[:-1]

  # don't include leading "/" from file name if present
  if member.filename[0] == '/':
    targetpath = os.path.join(targetpath, member.filename[1:])
  else:
    targetpath = os.path.join(targetpath, member.filename)

  targetpath = os.path.normpath(targetpath)

  # Create all upper directories if necessary.
  upperdirs = os.path.dirname(targetpath)
  if upperdirs and not os.path.exists(upperdirs):
    os.makedirs(upperdirs)

  if member.filename[-1] == '/':
    if not os.path.isdir(targetpath):
      os.mkdir(targetpath)
    return targetpath

  fp = open(targetpath, 'wb')
  fp.write(zip.read(member.filename))
  fp.close()

  return targetpath

def createConvertersXmlFile(outputFile, includeList=[] ):
  """
  Creates the converters.xml file that includes all the files required to
  update the configuration."""
  log_debug(fileScopeLogger,"creating converters file %s" % outputFile)
  bFoundOneFile = False
  conversionDoc = Document()

  # Create root node
  root = conversionDoc.createElement("root")
  root.setAttribute("xmlns:xi", "http://www.w3.org/2001/XInclude")
  conversionDoc.appendChild(root)

  for elem in includeList:
    if os.path.exists(elem):
      bFoundOneFile = True
      node = conversionDoc.createElement("xi:include")
      #xi:include require a very specific path syntax.
      #This replace \ with / and space with %20
      elem = elem.replace('\\', '/')
      elem = urllib.quote(elem, safe=':/')
      node.setAttribute("href", elem)
      fallback = conversionDoc.createElement("xi:fallback")
      node.appendChild(fallback)
      root.appendChild(node)

  if not bFoundOneFile:
    log_error(fileScopeLogger,"Converters include list contains no valid files")
    return False
  
  try:
    conversionFile = open(outputFile,"w")
    indent = "  "
    conversionDoc.writexml(conversionFile, "", indent, "\n")
    conversionFile.close()
  except IOError, e:
    log_error(
      fileScopeLogger,
      "unable to create converters.xml file in working directory. error=%s" \
      % e
    )
    return False

  return True

def removeFile(filePath):
  """Delete a file safely"""
  if os.path.exists(filePath):
    log_debug(fileScopeLogger, "Deleting file '%s'" % filePath)
    try:
      os.remove(filePath)
    except OSError, e:
      log_warn(fileScopeLogger, "Unable to delete file '%s'. Error=%s" %
                (filePath, e))
    return False

  return True

#-----------------------------------------------------------------------------#
# @method importCfg : Starts the Smart Update process.
# @param in_strCfgArchivePath: path to the configuration archive to be imported
# @return 0 : everything is fine
# @return 100 : standard application failure error code
# @return 101 : update succeeded with warnings
# @return 102 : invalid config file
# @return 103 : export.txt is missing from configuration archive
# @return 104 : configuration comes from a different platform (Linux or Windows)
# @return 105 : version of the configuration is greater then the
#               currently installed application version.
# @return 106 : currently installed version of the application is more then 
#               one major version of the configuration being imported 
# @return 107 : brand is not matching
# @return 108 : sub_brand is not matching
#-----------------------------------------------------------------------------#
def importCfg(in_strCfgArchivePath, in_strOutputDir, in_strBinPath=None,
              in_strWorkingDir=None, in_strXslScriptDir=None,
              in_strAppServiceName=''):
  log_info(
    fileScopeLogger,
    "Beginning import process of config %s" % in_strCfgArchivePath
  )
  log_debug(
    fileScopeLogger,
    "output dir=%s\nbin path=%s" % (in_strOutputDir, in_strBinPath)
  )

  #---------------------------------------------------------------------------#
  # Variables
  #---------------------------------------------------------------------------#
  if in_strWorkingDir:
    strWorkingDir = in_strWorkingDir
  else:
    strWorkingDir = \
      globalCfg.getParameter("netborder.smartUpdate.Working.Directory")

  strAddJobsScriptDirPath= os.path.dirname(os.path.abspath(__file__))
  strAddJobsScriptPath = os.path.join(strAddJobsScriptDirPath, "SmartUpdateAddJobs.py")
  strExtractionPath = ""
  dictCfgInfo = {}

  if in_strBinPath :
    strBinDir = in_strBinPath
  else:
    strBinDir = globalCfg.getParameter("netborder.Installation.Bin.Directory")

  strAppCmd       = ""
  strAppName      = ""
  strAppBrand     = ""
  strAppSubBrand  = ""
  strAppVersion   = ""
  strAppPlatform  = platform.system()
  # In Python 2.5, platform.system() reports "Microsoft" on Vista and up,
  # when "Windows" is expected, see http://bugs.python.org/issue1082
  if strAppPlatform == 'Microsoft':
    strAppPlatform = 'Windows'
  strAppInstDir   = globalCfg.getParameter("netborder.Installation.Directory")
  strAppCfg       = in_strOutputDir
  strAppServiceName = in_strAppServiceName

  if in_strXslScriptDir:
    strAppXslScript = in_strXslScriptDir
  else:
    strAppXslScript = \
      globalCfg.getParameter("netborder.smartUpdate.XslScripts.Directory")
  
  strCfgBrand     = ""
  strCfgSubBrand  = ""
  strCfgVersion   = ""
  strCfgPlatform  = ""
  strCfgInstDir   = ""
  strConvertersFilePath = strWorkingDir + "/converters.xml"

  smartUpdateStatus = JobStatus()
  xsltProc = XsltProcWrapper()
  returnCode = 0

  #---------------------------------------------------------------------------#
  # Implementation
  #---------------------------------------------------------------------------#
  # Path where the configuration archive will be extracted
  strExtractionPath = strWorkingDir + "/initialCfg"
  log_debug(fileScopeLogger,"extraction path=%s" % strExtractionPath)

  if not os.path.exists(in_strCfgArchivePath):
    log_error(
      fileScopeLogger, 
      "The configuration archive path '%s' is invalid" % in_strCfgArchivePath
    )
    return 102

  #Archive must be a zip file.
  elif not zipfile.is_zipfile(in_strCfgArchivePath):
    log_error(
      fileScopeLogger,
      "The configuration archive MUST be in the ZIP format for the "\
      "SmartUpdate to succeed."
    )
    return 102

  #Make sure that strExtractionPath directory is empty.
  if os.path.exists(strExtractionPath):
    log_debug(fileScopeLogger, "Deleting initial config directory '%s'" %
              strExtractionPath)
    try:
      shutil.rmtree(strExtractionPath)
    except (OSError, IOError), e:
      log_error(fileScopeLogger, "Unable to remove directory '%s'. Error=%s" %
                (strExtractionPath, e))
      return 100

  # Unzip the configuration
  cfgZip = zipfile.ZipFile(in_strCfgArchivePath)
  if not extractall(cfgZip, strExtractionPath):
    log_error(fileScopeLogger,"unable to extract the configuration archive")
    return 100

  # Get the application exe from the export.txt file
  try:
    exportFile = open(os.path.join(strExtractionPath,"export.txt"))
    for line in exportFile:
      line = line.split('=')
      log_debug(fileScopeLogger,"export.txt line=%s" % line)
      dictCfgInfo[line[0]] = line[1].strip('\n')
      log_debug(fileScopeLogger,"exportContent dict=%s" % dictCfgInfo)
    exportFile.close()
  except IOError, e:
    log_error(
      fileScopeLogger,
      "could not open file export.txt. unable to continue. error=%s" % e
    )
    return 103

  # Retrive all the information required in the extracted content of export.txt
  strAppName      = dictCfgInfo['application']
  strCfgBrand     = dictCfgInfo['brand']
  strCfgSubBrand  = dictCfgInfo['sub_brand']
  strCfgPlatform  = dictCfgInfo['platform']
  # In Python 2.5, platform.system() reports "Microsoft" on Vista and up
  # when "Windows" is expected, see http://bugs.python.org/issue1082
  # Correct config info since "Microsoft" was written by earlier versions of
  #  NBE
  if strCfgPlatform == 'Microsoft':
    strCfgPlatform = 'Windows'
  strCfgVersion   = dictCfgInfo['version']
  strCfgInstDir   = dictCfgInfo['installDir']
  strCfgAutoStart = ''
  strCfgRestartOnFail = ''

  #Make sure that the service info is in the export.txt file
  if 'automatic_start' in dictCfgInfo:
    strCfgAutoStart   = dictCfgInfo['automatic_start']

  if 'restart_failure' in dictCfgInfo:
    strCfgRestartOnFail   = dictCfgInfo['restart_failure']


  log_debug(
    fileScopeLogger,
    "Cfg App Name=%s, Cfg Brand=%s, Cfg SubBrand=%s, Cfg Platform=%s, " \
    "Cfg Version=%s, Cfg Install Dir=%s AutoStart=%s RestartOnFail=%s" \
    % (strAppName, strCfgBrand, strCfgSubBrand, strCfgPlatform, \
    strCfgVersion, strCfgInstDir, strCfgAutoStart, strCfgRestartOnFail)
  )

  # Validate the platform (Linux/Windows)
  if not strAppPlatform == strCfgPlatform:
    log_error(
      fileScopeLogger,
      "Trying to import a %s configuration on a %s platform." \
      % (strCfgPlatform, strAppPlatform)
    )
    return 104 

  # Get the application brand, sub_brand and version info
  import subprocess
  strAppCmd = \
    os.path.join(strBinDir, strAppName) #+ " --version"
  log_debug(fileScopeLogger, "applicationCmd=%s" % strAppCmd)

  appProcess = subprocess.Popen(args=[strAppCmd,"--version"],
                                stdin=subprocess.PIPE,
                                stdout=subprocess.PIPE,
                                stderr=subprocess.PIPE)

  stdout, stderr = appProcess.communicate()

  if stderr:
    log_error(
      fileScopeLogger,
      "failed to validate the application information. error=%s" % stderr
    )
    return 100
  
  if stdout == "": #stdout is a empty string
    log_error(
      fileScopeLogger,
      "%s --version returned with no output. The application might be "
      "corrupted." % self.appExePath
    )
    return 100

  log_debug(fileScopeLogger,"netborder-gateway ouptut: %s" % stdout)

  appInfoLines = stdout.split('\r\n')
  appInfo = appInfoLines[0].split()

  # At this point, appInfo is an array containing the following information
  # ['INFO', '-', 'brand', 'sub_brand', 'appType', 'version', 'versionNum']
  strAppBrand = appInfo[2].lower()
  strAppSubBrand = appInfo[3].lower()
  strAppVersion = appInfo[6].lower()

  if strCfgVersion > strAppVersion:
    log_error(
      fileScopeLogger,
      "The configuration version (%s) must be lower than the current" \
      "application version (%s)." % (strCfgVersion,strAppVersion)
    )
    return 105

  try:
    appMajorVersion = int(strAppVersion[0])
    cfgMajorVersion = int(strCfgVersion[0])
  except ValueError, e:
    log_error(
      fileScopeLogger,
      "Failed to convert major version to int. Error=%s" % e
    )
    return 100

  if (appMajorVersion - cfgMajorVersion) > 1 :
    log_error(
      fileScopeLogger,
      "The installed application version (%s) must be less than one "\
      "major version from the configuration being imported (%s)." %
      (strAppVersion, strCfgVersion)
    )
    return 106

  log_debug(
    fileScopeLogger,
    "App Brand=%s, App SubBrand=%s, App Version=%s" \
    % (strAppBrand, strAppSubBrand, strAppVersion )
  )

  # Validate the brand and sub brand matches
  if not strAppBrand == strCfgBrand:
    log_error(
      fileScopeLogger,
      "invalid brand: the brand of the configuration (%s) and the brand of " \
      "the installed appication (%s) must match." \
      % (strCfgBrand, strAffBrand)
    )
    return 107

  if not strAppSubBrand == strCfgSubBrand:
    log_error(
      fileScopeLogger,
      "invalid sub_brand: the sub_brand of the configuration (%s) and the " \
      "sub_brand of the installed appication (%s) must match." \
      % (strCfgSubBrand, strAppSubBrand)
    )
    return 108

  
  if strAppServiceName == '':
    log_debug(fileScopeLogger, "Retrieving application '%s' product name." %
              strAppCmd)
    appProcess = subprocess.Popen(args=[strAppCmd,"--product-name"], 
                                  stdin=subprocess.PIPE,
                                  stdout=subprocess.PIPE,
                                  stderr=subprocess.PIPE)

    stdout, stderr = appProcess.communicate()

    if stderr:
      log_error(
        fileScopeLogger,
        "Failed to retrieve application product name. error=%s" % stderr
      )
      return 100

    if platform.system() == "Linux":
      strAppServiceName = strAppName
    else:
      #Retrieve product name
      for line in stdout.split('\n'):
        m = re.search('Product name : (.+)' , line)
        if m:
          #Remove leading and trailing whitespace.
          strAppServiceName = m.group(1).strip()

    if strAppServiceName is None:
      smartUpdateStatus.addWarning("Failed to retrieve application service "\
                                   "name. Service information will not be "\
                                   "configured.")
    else:
      log_debug(fileScopeLogger, "Found application product name '%s'" %
                                  strAppServiceName)

  else:
    strAppServiceName = in_strAppServiceName

  #If fail to retrieve service name.
  if strAppServiceName == '':
    smartUpdateStatus.addWarning("Failed to retrieve application service "\
                                 "name. Service might not be configured as "\
                                 "it was before.")
  else:

    #Configure service as defined in export.txt.
    if strCfgAutoStart == '':
      smartUpdateStatus.addWarning("Could not retrieve service '%s' start up "\
                                    "type information. If the service was "\
                                    "configured to start automatically you "\
                                    "need to reconfigure it manually." % 
                                    strAppServiceName)
        
    elif strAppServiceName != '' and strCfgAutoStart == 'True':
      log_debug(
        fileScopeLogger,
        "Configuring service '%s' to start automatically." % strAppServiceName)

      if netborderPyThread.setServiceAutomaticStartup(strAppServiceName):
        log_info(fileScopeLogger, "Configured service '%s' to start "\
                "automatically on start up." % strAppServiceName)
      else:
        smartUpdateStatus.addWarning("Failed to configure service '%s' to "\
                                     "start automatically.")

    if strCfgRestartOnFail == '':
      smartUpdateStatus.addWarning("Could not retrieve service '%s' restart "\
                                    "on failure information. If the service "\
                                    "was configured to automatically restart "\
                                    "on failure you need to reconfigure it "\
                                    "manually." % strAppServiceName)

    elif strAppServiceName != '' and strCfgRestartOnFail == 'True':
      log_debug(
        fileScopeLogger,
        "Configuring service '%s' to restart on failure." % strAppServiceName)

      if netborderPyThread.\
          setServiceFailureAutomaticRestartCount(strAppServiceName, 2):
        log_info(fileScopeLogger, "Configured service '%s' to restart "\
                "automatically on failure." % strAppServiceName)
      else:
        smartUpdateStatus.addWarning("Failed to configure service '%s' to "\
                                     "restart on failure.")


  # Generate the xml file that includes both the smartUpdateConverters.xml and
  # patchConverter.xml files.
  # !+! jpboily : Bug 6654 - In the case where we are restoring a configuration
  # that has the same version, patchConverter.xml in the archive file should
  # not be considerated since it only brings the update process to a version
  # greater than the one currently installed.
  if strCfgVersion == strAppVersion:
    includeList = [strAppInstDir  + "/config/smartUpdateConverters.xml"]
  else:
    includeList = [
      strAppInstDir  + "/config/smartUpdateConverters.xml",
      strExtractionPath + "/patchConverter.xml"
    ]

  log_debug(fileScopeLogger,"converters include list=%s" % includeList)
  if not createConvertersXmlFile(strConvertersFilePath, includeList):
    log_error(fileScopeLogger,"unable to create the converters.xml file")
    return 100

  # Execute the xslt script that reads the generated converters.xml file and
  # creates the SmartUpdateJRunnable file
  parserXsl = strAppXslScript + "/smartUpdateConvertersParser.xsl"
  xsltProcOpts = [
    "--xinclude",
    "--stringparam initVersion %s" % strCfgVersion
  ]
  xsltProc.configure( parserXsl, strConvertersFilePath, strAddJobsScriptPath,
                      xsltProcOptions=xsltProcOpts )

  (rc, msg, warningList) = xsltProc.call()
  if not rc == 0:
    log_error(
      fileScopeLogger,
      "failed to parse converters.xml file. rc=%s, cause=%s." % (rc,msg))
    return 100

  if not warningList == []:
    log_warn(
      fileScopeLogger,
      "the parsing of the converters list succeeded, but the following things"\
      "needs to be verified by the user: %s " % warningList
    )
    returnCode = -1

  # Instantiate TempDirManager that manages the directories where files are
  # copied during the update process
  # Replace installation directory path and version string by the new version.
  replaceStringDict = {
    strCfgInstDir:strAppInstDir,
    strCfgVersion:strAppVersion
  }
  tmpDirMgr = \
    TempDirManager(strExtractionPath, strAppCfg, 
                   replaceStringDict, '.properties')
  tmpDirMgr.initialize()

  # Instantiate the SmartUpdateJobRunner that will execute all the conversion
  # jobs
  jobRunner = SmartUpdateJobRunner(smartUpdateStatus, tmpDirMgr)

  if jobRunner is None:
    log_error(fileScopeLogger, "failed to initialize the SmartUpdateJobRunner")
    return 100

  # Configure the SmartUpdateJobRunner
  if not jobRunner.configure():
    log_error(fileScopeLogger, "failed to configure the job runner.")
    return 100

  # Start the SmartUpdateJobRunner
  if not jobRunner.run():
    log_error(fileScopeLogger,"One of the conversion jobs failed.")
    return 100

  warnMsgList = smartUpdateStatus.getWarning()

  if smartUpdateStatus.hasWarning():
    #Update process succeeded but user might need to verify some setting.
    returnCode = 101
    if not warnMsgList == []:
      log_warn(fileScopeLogger, "Update process succeeded, but some setting " \
               "might need to be completed manually. Look for the following " \
               "logs for more information.")
      for warnMsg in warnMsgList:
        log_warn(fileScopeLogger, warnMsg)


  #Remove files created by the Import process.
  removeFile(strAddJobsScriptPath)
  removeFile(strAddJobsScriptPath + 'c')
  removeFile(strConvertersFilePath)

  #Remove initial config directory.
  if os.path.exists(strExtractionPath):
    log_debug(fileScopeLogger, "Deleting directory '%s'" % strExtractionPath)
    try:
      shutil.rmtree(strExtractionPath)
    except (OSError, IOError), e:
      log_warn(fileScopeLogger, "Unable to remove directory '%s'. Error=%s" %
                (strExtractionPath, e))

  #If working directory is empty remove it.
  if not os.listdir(strWorkingDir):
    log_debug(fileScopeLogger, "Deleting directory '%s'" % strWorkingDir)
    try:
      os.rmdir(strWorkingDir)
    except OSError, e:
      log_warn(fileScopeLogger, "Unable to remove directory '%s'. Error=%s" %
                (strWorkingDir, e))

  return returnCode


if __name__ == "__main__":

  def runtest(cfgArchivePath):
    instDir = globalCfg.getParameter("netborder.Installation.Directory")
    rc = importCfg(cfgArchivePath, instDir + "/config")
    print "importCfg returned with code [%s]" % rc

  import sys
  args = sys.argv[1:]
  if len(args) == 1:
    print "args[0]=%s" % args[0]
    runtest(args[0])
  else:
    #Check if the current os is windows (nt) else it's linux.
    if os.name == 'nt':
      archive = "test-input/nbe_config_3.0.0_windows.zip"
    else:
      archive = "test-input/nbe_config_4.1.1_linux.zip"
    print ("Running with archive: %s" % archive)
    runtest(archive)

## !!MIRROR FILE, DO NOT MODIFY!!
