#! /usr/bin/env python
#
# Author: Jean-Marc Legrand
# Date:   Tue May 25 2010
#
# Copyright (C) 2010 Paraxip Technologies
# All Rights Reserved.
#
# This code is Paraxip Technologies Confidential Property.
# Use of and Access to this code is covered by a previously executed
# Non-disclosure Agreement between Paraxip Technologies and the Recipient.
# This code is being supplied for evaluation purposes only and is not to be
# used for any other purpose.

import sys
import os
import datetime
import time

# Imports for SQLalachemy
from sqlalchemy import Integer, String, DateTime, Boolean
from sqlalchemy import Column, ForeignKey
from sqlalchemy import and_
from sqlalchemy import create_engine
from sqlalchemy import exceptions as dbexcep
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import relation, backref
from sqlalchemy.orm import sessionmaker

# Force declaration of class attributes in __init__ method
__pychecker__ = "initattr"

# Init logging
from paraxip.infra.PyLoggingFunctions import *
from paraxip.infra.Assert import *
fileScopeLogger = Logger("netborder.callData")


def normalizeDBUrl( in_strDBUrl ):
  __pychecker__ = 'unusednames=t'
  t = TraceScope(fileScopeLogger, "normalizeDBUrl")
  
  strNormalizedDBUrl = in_strDBUrl

  # SQLITE normalization of absolute path on Windows e=
  # create_engine('sqlite:///C:\\path\\to\\database.db') if the URL
  # contains "<drive>:/" we have to replace all '/' starting at
  # '<drive>:/' by '\'
  if  in_strDBUrl.find( "sqlite:" ) == 0 or  in_strDBUrl.find( "SQLITE:" ) == 0:
    tokBeforeDriveLetter = ":///"
    startReplaceAt = in_strDBUrl.find( tokBeforeDriveLetter )
    if startReplaceAt != -1:
      startReplaceAt += len(tokBeforeDriveLetter)
      if in_strDBUrl[startReplaceAt+1: startReplaceAt+2] == ':':
        endReplaceAt = len(in_strDBUrl)
        strNormalizedDBUrl = "sqlite:///" + \
                             in_strDBUrl[startReplaceAt:endReplaceAt].replace('/', '\\' )
  log_info( fileScopeLogger, "strNormalizedDBUrl=%s", strNormalizedDBUrl )
  return strNormalizedDBUrl    

# Return the current 15 minutes interval since january 1st 1970
def getCurrent15minutesInterval():
  # 900 seconds in 15 minutes
  return time.time() / 900

# Use the declarative_base for all classes that need to be stored in
# the DB
Base = declarative_base()

# This class contains a call parameter such ANI, DNIS, SIP REQUEST URI
# associated with a given event.
class CallParameter(Base):
  
  __tablename__ = 'callParameter'
  
  m_id = Column( Integer, primary_key=True )
  m_name = Column(String(100)) 
  m_value = Column(String)
  m_callEventId = Column(Integer, ForeignKey('callDataEvent.m_id'))
     
  def __init__(self, in_name, in_value):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "CallParameter.__init__")
    
    self.m_name = in_name
    self.m_value = in_value
    
  def __repr__(self):
    return  "<CallParameter('%s', '%s', '%s', '%s')>" % \
           (self.m_id, self.m_name, self.m_value, self.m_callEventId )

  def __str__(self):
    return "%s=%s" % (self.m_name, self.m_value)

class CallDataEvent(Base):
  
  __tablename__ = 'callDataEvent'
  
  m_id = Column( Integer, primary_key=True )
  m_timestamp = Column(DateTime)
  m_interval = Column(Integer, index=True)
  m_type = Column(String(20))
  m_newState = Column(String(20), index=True)
  m_direction = Column(String(20))
  m_callDataId = Column(Integer, ForeignKey('callData.m_id'))
  m_callParameters = relation(CallParameter, order_by=CallParameter.m_id, \
                              backref='callDataEvent', \
                              cascade="all, delete, delete-orphan" )
     
  def __init__(self):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "CallDataEvent.__init__")
    
    self.m_timestamp = datetime.datetime.now()
    self.m_interval = getCurrent15minutesInterval()
    self.m_type = None
    self.m_state = None
    self.m_direction = None
    
  def __repr__(self):
    return  "<CallDataEvent('%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s')>" % \
           (self.m_id, self.m_timestamp, sefl.m_interval, self.m_type, \
            self.m_newState, self.m_direction, self.m_callParameters, \
            self.m_callDataId )

  def setCallParameter(self, in_name, in_value):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "CallDataEvent.setCallParameter")
    try:
      # Find any exist call info matching
      existingCallParameter = None
      for parameter in self.m_callParameters:
        if parameter.m_name == in_name:
          existingCallParameter = parameter
          break
      
      # Add call info if it does not exist or just change its value
      if existingCallParameter is None:      
        self.m_callParameters.extend( [ CallParameter( in_name, in_value ) ] )
      else :
        existingCallParameter.m_value = in_value
        
      return True
    
    except:
      self.m_session.rollback()
      log_error( fileScopeLogger,
                 "failed to set call parameter '%s' due to the following error '%s'",
                 in_name, sys.exc_info()[0] )
      return False
    
  
class CallData(Base):
  
  __tablename__ = 'callData'

  m_id = Column( Integer, primary_key=True )
  m_startTimestamp = Column(DateTime)
  m_endTimestamp = Column(DateTime)
  m_startInterval = Column(Integer, index=True)
  m_callId = Column(String(50), unique=True)
  m_active = Column(Boolean)
  m_state = Column(String(30), index=True)
  m_direction = Column(String(15))
  m_events = relation(CallDataEvent, order_by=CallDataEvent.m_timestamp, \
                      backref='callData', \
                      cascade="all, delete, delete-orphan")
  
  def __init__(self):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "CallData.__init__")

    self.m_startTimestamp = datetime.datetime.now()
    self.m_startInterval = getCurrent15minutesInterval()
    self.m_endTimestamp = None
    self.m_active = True
    self.m_callId = None
    self.m_state = None
    self.m_registry = None
    self.m_direction = None

  def __repr__(self):
    return "<CallData('%s', '%s', '%s', '%s', '%s', '%r', '%s', '%s')>" % \
           (self.m_id, self.m_startTimestamp, self.m_startInterval, \
            self.m_endTimestamp, \
            self.m_callId, self.m_active, \
            self.m_state, self.m_events)

  def pushEvent(self, in_event):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "CallData.pushEvent")
    try:
      log_debug( fileScopeLogger, 
                 "pushEvent '%s' to call '%s'",
                 in_event.m_type.encode('ascii','ignore'),
                 self.m_callId.encode('ascii','ignore') )
    
      if in_event.m_newState != self.m_state :
        if self.m_state is None:
          log_debug( fileScopeLogger,
                     "state change None --> '%s'",
                     in_event.m_newState.encode('ascii','ignore') )
        else:        
          log_debug( fileScopeLogger,
                     "state change '%s' --> '%s'",
                     self.m_state.encode('ascii','ignore'),
                     in_event.m_newState.encode('ascii','ignore') )
        
      if self.m_direction is None:
        self.m_direction = in_event.m_direction

      self.m_state = in_event.m_newState    
      self.m_events.extend( [ in_event ] )
      #self.m_registry.commit()

      return True
      
    except:
      self.m_session.rollback()
      log_error( fileScopeLogger,
                 "failed to pushEvet '%s' due to the following error '%s'",
                 in_event, sys.exc_info()[0] )
      return False

  def callEnd(self):    
    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "CallData.callEnd")
    try:
      self.m_active = False
      self.m_endTimestamp = datetime.datetime.now()
      #self.m_registry.commit()
      log_debug( fileScopeLogger,
                 "ending call '%s'", self.m_callId.encode('ascii','ignore') )
      return
    
    except:
      self.m_session.rollback()
      log_error( fileScopeLogger,
                 "failed to end call '%s' due to the following error '%s'",
                 self, sys.exc_info()[0] )

class CallDataRegistry:

  def __init__(self):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "CallDataRegistry.__init__")    
    self.m_callDataDB = None
    self.m_metaData = None
    self.m_Session = None
    self.m_session = None

  # Open DB URL
  #
  # SQLITE Connect Strings
  # ----------------------
  #
  # The file specification for the SQLite database is taken as the
  # database portion of the URL. Note that the format of a url is:
  #
  # driver://user:pass@host/database
  #
  # This means that the actual filename to be used starts with the
  # characters to the right of the third slash.
  #
  # * relative path
  #
  #   e = create_engine('sqlite:///path/to/database.db')
  #
  # * absolute path
  #
  #   e = create_engine('sqlite:////path/to/database.db')
  #
  # An absolute path, which is denoted by starting with a slash,
  # means you need four slashes:
  #
  # To use a Windows path, regular drive specifications and
  # backslashes can be used. Double backslashes are probably needed:
  #
  # * absolute path on Windows
  #
  #  e = create_engine('sqlite:///C:\\path\\to\\database.db')
  #
  #  The sqlite :memory: identifier is the default if no filepath is
  #  present. Specify sqlite:// and nothing else:
  #
  # * in-memory database
  #
  #  e = create_engine('sqlite://')
  #
  # Not supported by Paraxip CallData because we need multithread support
  #
  def configure(self, in_strCallDataDBUrl, in_fDebugSql=False,
                in_fFixDB=True):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "CallDataRegistry.configure")

    # Pysqlite connections do not support being moved between threads,
    # unless the check_same_thread Pysqlite flag is set to False.
    try:

      # open database
      strDBUrl = normalizeDBUrl(in_strCallDataDBUrl)
      self.m_callDataDB = create_engine(strDBUrl, echo=in_fDebugSql)
      if self.m_callDataDB is None:
        log_error( fileScopeLogger,
                   "failed to open call data database: '%s'",
                   in_strCallDataDBUrl )
        return False

      #
      # tune database parameter to get high performance
      #

      # Query or change the setting of the "synchronous" flag. The
      # first (query) form will return the synchronous setting as an
      # integer. When synchronous is FULL (2), the SQLite database
      # engine will pause at critical moments to make sure that data
      # has actually been written to the disk surface before
      # continuing. This ensures that if the operating system crashes
      # or if there is a power failure, the database will be
      # uncorrupted after rebooting. FULL synchronous is very safe,
      # but it is also slower. When synchronous is NORMAL (1), the
      # SQLite database engine will still pause at the most critical
      # moments, but less often than in FULL mode. There is a very
      # small (though non-zero) chance that a power failure at just
      # the wrong time could corrupt the database in NORMAL mode. But
      # in practice, you are more likely to suffer a catastrophic disk
      # failure or some other unrecoverable hardware fault. With
      # synchronous OFF (0), SQLite continues without pausing as soon
      # as it has handed data off to the operating system. If the
      # application running SQLite crashes, the data will be safe, but
      # the database might become corrupted if the operating system
      # crashes or the computer loses power before that data has been
      # written to the disk surface. On the other hand, some
      # operations are as much as 50 or more times faster with
      # synchronous OFF.
      
      # PRAGMA synchronous = 0 | OFF | 1 | NORMAL | 2 | FULL;      
      self.m_callDataDB.execute('pragma synchronous=0')

      # The temp_store values specifies the type of database back-end
      # to use for temporary files. The use of a memory database for
      # temporary tables can produce signifigant savings. DEFAULT
      # specifies the compiled-in default, which is FILE unless the
      # source has been modified.

      # PRAGMA temp_store = DEFAULT (0), FILE (1), and MEMORY (2).
      self.m_callDataDB.execute('pragma temp_store=2')

      # When the count_changes setting is ON, the callback function is
      # invoked once for each DELETE, INSERT, or UPDATE operation. The
      # argument is the number of rows that were changed. If you don't
      # use this feature, there is a small speed increase from turning
      # this off.

      # PRAGMA count_changes=OFF(0)
      self.m_callDataDB.execute('pragma count_changes=0')

      # Create tables
      self.m_metaData = Base.metadata
      self.m_metaData.create_all(self.m_callDataDB)
      self.m_Session = sessionmaker(bind=self.m_callDataDB)
      self.m_session = self.getNewDBSession()

      # Fix DB
      if in_fFixDB is True:
        self.fixCallDataDB();

      # Success
      return True
    
    except dbexcep.DBAPIError, e:
      log_error( fileScopeLogger,
                 "failed open CallData database '%s' due to the following error: %s",
                 strDBUrl, e )
      return False

  def onControlThreadChange(self):
    ''' A session object is only valid in the thread where the session
    was created.  This is a limitation of the SQLITE... The C++
    application that uses this module that call this method in the
    onSvnEntry() method to and makere sure to call any objects created
    with a session for the whole life of that object'''
    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "CallDataRegistry.onControlThreadChange")
    self.m_session = self.getNewDBSession()

  def addCallData(self, in_callData ):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "CallDataRegistry.addCallData")

    log_debug( fileScopeLogger,
               "adding call '%s' to DB",
               in_callData.m_callId.encode('ascii','ignore') )

    in_callData.m_registry = self
    try:
      self.m_session.add(in_callData)
      log_debug( fileScopeLogger, "commiting call data '%s'",
                 in_callData.m_callId.encode('ascii','ignore') )
      #self.m_session.commit()
      return True
    except:
      log_error( fileScopeLogger,
                 "failed to add CallData '%s' due to the following error '%s'",
                 in_callData, sys.exc_info()[0] )
      self.m_session.rollback()
      return False

  def getNewDBSession(self):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "CallDataRegistry.getNewDBSession")
    return self.m_Session()

  def commit(self):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "CallDataRegistry.commit")
    return self.m_session.commit()

  def rollback(self):
    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "CallDataRegistry.rollback")
    return self.m_session.rollback()

  def fixCallDataDB(self ):
    ''' Because the gateway can be killed by the user or can crash,
    the database may contains active call data that need to be marked
    inactive.  This method fix the call data active state.''' 
    __pychecker__ = 'unusednames=t'
    t = TraceScope(fileScopeLogger, "CallDataRegistry.cleanup")

    session = self.getNewDBSession()
    activeCallsToFix = self.getActiveCalls(session)
    log_info( fileScopeLogger,
               "fixing active call data entries that are not active anymore '%s'", \
               activeCallsToFix )

    for call in activeCallsToFix:      
      call.m_active = False
      
    session.commit()

  def getAllCallsQuery(self, in_session=None):
    session = self.m_session if in_session is None else in_session        
    return session.query(CallData)

  def getAllCalls(self, in_session=None):
    return self.getAllCallsQuery().all()

  def getActiveCallsQuery(self, in_session=None):
    session = self.m_session if in_session is None else in_session
    return session.query(CallData).\
           filter(CallData.m_active == True)

  def getActiveCalls(self, in_session=None):
    return self.getActiveCallsQuery(in_session).all()
    
  def getInactiveCallsQuery(self, in_session=None):
    session = self.m_session if in_session is None else in_session
    return session.query(CallData).\
           filter(CallData.m_active == False)

  def getInactiveCalls(self, in_session=None):
    return self.getInactiveCallsQuery(in_session).all()

  def getCallsInTimeRangeFromQuery(self, in_query,
                                   in_startTime=None,
                                   in_endTime=None ):    
    newQuery = in_query
    if in_endTime is not None:
      newQuery = newQuery.filter( CallData.m_endTimestamp <= in_endTime)

    if in_startTime is not None:
      newQuery = newQuery.filter( CallData.m_startTimestamp >= in_startTime)

    return newQuery

  def getSameDirectionDataFromQuery(self,in_query,in_strDirection):
    return in_query.filter(CallData.m_direction.contains(in_strDirection))

  def getNumActiveLongCalls(self,in_session=None):
    if in_session is None:
      session = self.m_session
    else:
      session = in_session 
    datetime1 = datetime.datetime.now() - datetime.timedelta(hours=4)
    return session.query(CallData).\
           filter(CallData.m_active == True).\
           filter( CallData.m_startTimestamp  <= datetime1).count()

  def getDataInitiatedInInterval(self,
                                 in_strIntervalStartTime,
                                 in_strIntervalLenghtSeconds,
                                 in_strDataType,
                                 in_session=None):
    session = self.m_session if in_session is None else in_session
    intervalStart = datetime.datetime.\
                    strptime(in_strIntervalStartTime[:19],"%Y-%m-%d %H:%M:%S")
    intervalLenght = long(in_strIntervalLenghtSeconds)
    lim =  intervalStart + datetime.timedelta(seconds=intervalLenght)

    if in_strDataType == "CallDataEvents":
      return session.query(CallDataEvent).\
             filter( CallDataEvent.m_timestamp >= intervalStart ).\
             filter( CallDataEvent.m_timestamp  <= lim)
    else:
      return session.query(CallData).\
             filter( CallData.m_startTimestamp >=  intervalStart).\
             filter( CallData.m_startTimestamp  <= lim)

  def getCallDataTerminatedInInterval(self,
                                      in_strIntervalStartTime,
                                      in_strIntervalLenghtSeconds,
                                      in_session=None):
    session = self.m_session if in_session is None else in_session
    intervalStart = datetime.datetime.\
                    strptime(in_strIntervalStartTime[:19],"%Y-%m-%d %H:%M:%S")
    intervalLenght = long(in_strIntervalLenghtSeconds)
    lim =  intervalStart + datetime.timedelta(seconds=intervalLenght)
    return session.query(CallData).\
           filter( CallData.m_endTimestamp != None).\
           filter( CallData.m_endTimestamp >=  intervalStart).\
           filter( CallData.m_endTimestamp <= lim)
       
          
if __name__ == '__main__':

  # Main
  def main(argv=None):

    if argv is None:    
      argv = sys.argv

    fileScopeLogger.setLogLevel(LogLevel.info)

    #Test getNormalizeDBUrl
    strTest = "sqlite:///c:/a/c/d"
    if normalizeDBUrl(strTest) != "sqlite:///c:\\a\\c\\d":
      log_error( fileScopeLogger, "failed to normalize '%s'", strTest )
      return -1

    strTest = "sqlite:////a/c/d"
    if normalizeDBUrl(strTest) != "sqlite:////a/c/d":
      log_error( fileScopeLogger, "failed to normalize '%s'", strTest )
      return -1

    strTest = "SQLITE:///c:/a/c/d"       
    if normalizeDBUrl(strTest) != "sqlite:///c:\\a\\c\\d":
      log_error( fileScopeLogger, "failed to normalize '%s'", strTest )
      return -1

    strTest = "SQLITE:////a/c/d"
    if normalizeDBUrl(strTest) != "SQLITE:////a/c/d":
      log_error( fileScopeLogger, "failed to normalize '%s'", strTest )
      return -1

    strDBFileName = "./callData.db"
    
   ## try:
##      os.remove( strDBFileName )
##      log_info( fileScopeLogger, "deleted file %s", strDBFileName )
##    except OSError, e:
##      log_info( fileScopeLogger, "can't delete file %s because %s",
##                strDBFileName, e )
    
    callDataRegistry = CallDataRegistry()
    if callDataRegistry.configure('sqlite:///' + strDBFileName,
                                  True ) == False:
      return 1
    
#    if callDataRegistry.configure('sqlite://',
#                                  False ) == False:
#      return 1


    loopStart = time.clock()
    testDuration = 7200
    elapsedtime = 0
    numCalls = 0

    while elapsedtime <  testDuration:

      #callDataRegistry.m_session.begin()
      call1 = CallData()
      call1.m_callId = '%s-12345-%s' % (os.getpid(), numCalls)
      
      call1.m_active = True
      if callDataRegistry.addCallData(call1) == False:
        return 2

      event1 = CallDataEvent()
      event1.m_type = 'info'
      event1.m_direction = 'SIP'
      event1.m_newState = 'SETUP'
      event1.setCallParameter( "pstn.in.ani", '4504417377' )
      event1.setCallParameter( "pstn.in.ani", '4504417378' )
      event1.setCallParameter( "pstn.in.dnis", '4504411234' )
      call1.pushEvent( event1 )
      
      event2 = CallDataEvent()
      event2.m_type = 'connect'
      event2.m_direction = 'SIP OUT'
      event2.m_newState = 'CONNECTED'
      call1.pushEvent( event2 )

      event3 = CallDataEvent()
      event3.m_type = 'disconnect'
      event3.m_direction = 'SIP OUT'
      event3.m_newState = 'DISCONNECTED'
      event3.setCallParameter( "reason", 'NORMAL' )
      call1.pushEvent( event3 )
      call1.callEnd()

      callDataRegistry.commit()

      break

      dEnd = time.clock()
      elapsedtime =  dEnd - loopStart
      #log_info( fileScopeLogger, "elapsed time %s sec" ,elapsedtime)
      numCalls = numCalls + 1

      if (numCalls % 250) == 0:
        callDataRegistry.commit()
        log_info( fileScopeLogger, "Call data rate %s call-data-entries/s" ,
                  numCalls/elapsedtime)              

    log_info( fileScopeLogger, "Call data rate %s call-data-entries/s" ,
              numCalls/elapsedtime)
    

    # Add call data
   ## log_info( fileScopeLogger, "creating call1..." )
##    call1 = CallData()
##    call1.m_callId = '12345'
##    call1.m_active = True
##    if callDataRegistry.addCallData(call1) == False:
##      return 2

##    log_info( fileScopeLogger, "setting event1 to call1..." )
##    event1 = CallDataEvent()
##    event1.m_type = 'setup'
##    event1.m_direction = 'PSTN IN==>SIP OUT'
##    event1.m_newState = 'SETUP'
##    event1.setCallParameter( "pstn.in.ani", '4504417377' )
##    event1.setCallParameter( "pstn.in.ani", '4504417378' )
##    event1.setCallParameter( "pstn.in.dnis", '4504411234' )
##    call1.pushEvent( event1 )

##    event2 = CallDataEvent()
##    event2.m_type = 'connect'
##    event2.m_direction = 'PSTN IN==>SIP OUT'
##    event2.m_newState = 'CONNECTED'
##    call1.pushEvent( event2 )

##    #call1.callEnd()
##    #return 0

##    # sleep a little
##    sleepDuration = 5
##    log_info(fileScopeLogger, "sleeping %d seconds...", sleepDuration )
##    time.sleep( sleepDuration )

##    # New call
##    log_info( fileScopeLogger, "creating call2..." )
##    call2 = CallData()
##    call2.m_callId = '12346'
##    call2.m_active = True
##    if callDataRegistry.addCallData(call2) == False:
##      return 3
    
##    log_info( fileScopeLogger, "setting event2 to call2..." )
##    event2 = CallDataEvent()
##    event2.m_type = 'setup'
##    event2.m_direction = 'SIP IN==>PSTN OUT'
##    event2.m_newState = 'SETUP'
##    event2.setCallParameter( "sip.in.fromUri", 'sip:jean@paraxip.com' )
##    event2.setCallParameter( "sip.in.toUri", 'sip:roger@paraxip.com' )
##    call2.pushEvent( event2 )
##    callDataRegistry.commit()

##    log_info( fileScopeLogger,
##              "**** all calls '%s'" % callDataRegistry.getAllCalls() )


##    # stop the first call
##    log_info( fileScopeLogger, "*** endding call1..." )
##    call1.callEnd()
##    call1 = None
##    log_info( fileScopeLogger, 
##              "**** active calls after endding call1 '%s'",
##              callDataRegistry.getActiveCalls() )

##    log_info( fileScopeLogger,
##              "**** inactive calls after endding call1 '%s'",
##              callDataRegistry.getInactiveCalls() )

##    # stop the 2nd call
##    log_info( fileScopeLogger, "*** endding call2..." )              
##    call2.callEnd()
##    call2 = None
##    log_info( fileScopeLogger,
##              "**** active calls after endding call2 '%s'", 
##              callDataRegistry.getActiveCalls() )

##    log_info( fileScopeLogger,
##              "**** all calls after endding all calls '%s'", 
##              callDataRegistry.getAllCalls() )


##    # Cleanup load entries
##    log_info( fileScopeLogger,
##              "**** all calls after cleanup'%s'",
##              callDataRegistry.getAllCalls() )

    # Success
    return 0

  # Execute main()
  sys.exit( main() )
## !!MIRROR FILE, DO NOT MODIFY!!
